@@ -1,9 +1,11 @@
 //! A reader default uses system endianness
 //! If you will use another endianness,use set_endian.
 
+use crate::Endian;
+use std::io::Seek;
+use std::io::SeekFrom;
 use std::io::Error;
 use std::io::ErrorKind;
-use crate::Endian;
 
 #[cfg(feature="codec")]
 use encoding_rs::*;
@@ -39,22 +41,7 @@ pub trait BinaryReader {
     /// 
     /// Assumed to be used for header checks.
     /// 
-    /// ```
-    /// use bin_rs::reader::*;
-    /// use std::io::Error;
-    /// fn test() ->  Result<(),Error> {
-    ///    let buffer = b"Hello World!";
-    ///    let mut reader = BytesReader::new(buffer);
-    ///    let buffer1 = reader.read_bytes_no_move(4)?;
-    /// // assert_eq!(buffer1,b"Hell");
-    ///    let buffer1 = reader.read_bytes_as_vec(4)?;
-    /// // assert_eq!(buffer1,b"Hell");
-    ///    let buffer1 = reader.read_bytes_as_vec(4)?;
-    /// // assert_eq!(buffer1,b"o Wo");
-    ///    return Ok(())
-    /// }
-    /// ```
-    /// 
+
     fn read_bytes_no_move(&mut self,len: usize) -> Result<Vec<u8>,Error>;
 
     fn read_u16(&mut self) -> Result<u16,Error>;
@@ -96,22 +83,7 @@ pub trait BinaryReader {
 
     /// read_ascii_string for C like ascii string.This function finishes find end marker 0x00.
     /// If reader read until \0, but skip size byte.
-    /// 
-    /// Ex)
-    /// 
-    /// ```
-    /// use bin_reader::reader::*;
-    /// use std::io::Error;
-    /// 
-    /// fn test() -> Result<String,Error> {
-    ///   let buffer = b"Hello World!\01234";
-    ///   let mut reader = BytesReader::new(buffer);
-    ///   let r = reader.read_ascii_string("Hello World!\01234".len())?;  // after \0 is trim
-    ///   //assert_eq!(r ,"Hello World!");
-    ///   return Ok(r)
-    /// }
-    /// ```
-    /// 
+
     fn read_ascii_string(&mut self,size:usize) -> Result<String,Error>;
 
     fn read_utf8_string(&mut self,size:usize) -> Result<String,Error>;
@@ -120,7 +92,9 @@ pub trait BinaryReader {
     fn read_local_string(&mut self,size:usize,code: CodeType) -> Result<String,Error>;
 
     /// skip size byte
-    fn skip_ptr(&mut self,size:usize) -> Result<usize,Error>; 
+    fn skip_ptr(&mut self,size:usize) -> Result<usize,Error>;
+
+    fn offset(&mut self) -> Result<u64,Error>;
 }
 
 /// BytesReader from creating Slice &\[u8\] or Vec<u8>,
@@ -158,7 +132,6 @@ impl BytesReader {
             ptr: 0,
             endian: crate::system_endian(),
         } 
-
     }
 
     fn check_bound(&mut self,size:usize) -> Result<(),Error> {
@@ -169,9 +142,72 @@ impl BytesReader {
             Ok(())
         }
     }
+
+    pub fn read_bytes(&mut self,len: usize) -> Result<&[u8],Error>{
+        self.check_bound(len)?;
+        Ok(&self.buffer[self.ptr..self.ptr+len])
+    }
 }
 
+
+impl Seek for BytesReader {
+    fn seek(&mut self, seek: SeekFrom) -> std::result::Result<u64, Error> {
+        match seek {
+            SeekFrom::Start(pos)=>{
+                if pos >= usize::MAX as u64 {
+                    let s = format!("BytesReader max offset is usize length but set {}",pos);
+                    return Err( Error::new(ErrorKind::Other, s))
+                } else if pos >= self.buffer.len() as u64 {
+                    let s = format!("set offset {},but buffer length is{}",pos,self.buffer.len());
+                    return Err( Error::new(ErrorKind::Other, s))
+                }
+                self.ptr = pos as usize;
+                Ok(self.ptr as u64)
+            },
+            SeekFrom::End(pos)=>{
+                if pos >= usize::MAX as i64 {
+                    let s = format!("BytesReader max offset is usize length but set {}",pos);
+                    return Err( Error::new(ErrorKind::Other, s))
+                } else if self.buffer.len() as i64 + pos < 0 || pos >= self.buffer.len() as i64 {
+                    let s = format!("set offset {},but buffer length is{}",pos,self.buffer.len());
+                    return Err( Error::new(ErrorKind::Other, s))
+                }
+                self.ptr = self.buffer.len() + pos as usize;
+                Ok(self.ptr as u64)
+            },
+            SeekFrom::Current(pos)=>{
+                let ptr = (self.ptr as i64) + pos;
+                if ptr >= usize::MAX as i64 {
+                    let s = format!("BytesReader max offset is usize length but set {}",ptr);
+                    return Err( Error::new(ErrorKind::Other, s))
+                } else if self.buffer.len() as i64 <= ptr || ptr < 0 {
+                    let s = format!("set offset {},but buffer length is{}",ptr,self.buffer.len());
+                    return Err( Error::new(ErrorKind::Other, s))
+                }
+                self.ptr = ptr as usize;
+                Ok(self.ptr as u64)
+            },
+        }
+    }
+}
+
+
+#[cfg(feature="stream")]
+impl<R:BufRead + Seek> StreamReader<R> {
+    pub fn new(reader: R) -> StreamReader<R> {
+        StreamReader {
+            reader: reader,
+            endian: crate::system_endian(),
+        }
+    }
+}
+
+#[cfg(feature="stream")]
 impl BinaryReader for BytesReader {
+    fn offset(&mut self) -> Result<u64,Error> {
+        return self.seek(SeekFrom::Current(0))
+    }
+
     fn set_endian(&mut self, endian: Endian) {
         self.endian = endian;
     }
@@ -206,6 +242,22 @@ impl BinaryReader for BytesReader {
     }
 
     // This function read bytes, but it does not move pointer.
+    /// ```
+    /// use bin_rs::reader::*;
+    /// use std::io::Error;
+    /// fn test() ->  Result<(),Error> {
+    ///    let buffer = b"Hello World!";
+    ///    let mut reader = BytesReader::new(buffer);
+    ///    let buffer1 = reader.read_bytes_no_move(4)?;
+    /// // assert_eq!(buffer1,b"Hell");
+    ///    let buffer1 = reader.read_bytes_as_vec(4)?;
+    /// // assert_eq!(buffer1,b"Hell");
+    ///    let buffer1 = reader.read_bytes_as_vec(4)?;
+    /// // assert_eq!(buffer1,b"o Wo");
+    ///    return Ok(())
+    /// }
+    /// ```
+    /// 
     fn read_bytes_no_move(&mut self, len: usize) -> Result<Vec<u8>, Error> {
         self.check_bound(len)?;
         let mut c:Vec<u8> = Vec::new();
@@ -221,7 +273,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_u16_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u16_le()
             }
         }
@@ -232,7 +284,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_u32_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u32_le()
             }
         }
@@ -243,7 +295,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_u64_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u64_le()
             }
         }
@@ -254,7 +306,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_u128_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u128_le()
             }
         }
@@ -265,7 +317,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_i16_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i16_le()
             }
         }
@@ -276,7 +328,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_i32_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i32_le()
             }
         }
@@ -287,7 +339,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_i64_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i64_le()
             }
         }
@@ -298,7 +350,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_i128_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i128_le()
             }
         }
@@ -309,7 +361,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_f32_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_f32_le()
             }
         }
@@ -320,7 +372,7 @@ impl BinaryReader for BytesReader {
             Endian::BigEndian => {
                 self.read_f64_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_f64_le()
             }
         }
@@ -551,6 +603,19 @@ impl BinaryReader for BytesReader {
     }
 
     /// read until \0, but skip size byte
+    /// ```
+    /// use bin_rs::reader::*;
+    /// use std::io::Error;
+    /// 
+    /// fn test() -> Result<String,Error> {
+    ///   let buffer = b"Hello World!\01234";
+    ///   let mut reader = BytesReader::new(buffer);
+    ///   let r = reader.read_ascii_string("Hello World!\01234".len())?;  // after \0 is trim
+    ///   //assert_eq!(r ,"Hello World!");
+    ///   return Ok(r)
+    /// }
+    /// ```
+    /// 
     fn read_ascii_string(&mut self,size:usize) -> Result<String,Error>{
         self.check_bound(size)?;
         let ptr = self.ptr;
@@ -611,20 +676,11 @@ impl BinaryReader for BytesReader {
 }
 
 
-#[cfg(feature="stream")]
-impl<R:BufRead> StreamReader<R> {
 
 
-    pub fn new(reader: R) -> StreamReader<R> {
-        StreamReader {
-            reader: reader,
-            endian: crate::system_endian(),
-        }
-    }
-}
-
 #[cfg(feature="stream")]
-impl<R:BufRead> BinaryReader for StreamReader<R> {
+impl<R:BufRead+Seek> BinaryReader for StreamReader<R> {
+
     fn set_endian(&mut self, endian: Endian) {
         self.endian = endian;
     }
@@ -669,7 +725,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_u16_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u16_le()
             }
         }
@@ -680,7 +736,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_u32_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u32_le()
             }
         }
@@ -691,7 +747,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_u64_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u64_le()
             }
         }
@@ -702,7 +758,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_u128_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_u128_le()
             }
         }
@@ -713,7 +769,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_i16_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i16_le()
             }
         }
@@ -724,7 +780,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_i32_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i32_le()
             }
         }
@@ -735,7 +791,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_i64_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i64_le()
             }
         }
@@ -746,7 +802,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_i128_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_i128_le()
             }
         }
@@ -757,7 +813,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_f32_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_f32_le()
             }
         }
@@ -768,7 +824,7 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
             Endian::BigEndian => {
                 self.read_f64_be()
             },
-            Endian::LittleEndtian => {
+            Endian::LittleEndian => {
                 self.read_f64_le()
             }
         }
@@ -971,4 +1027,15 @@ impl<R:BufRead> BinaryReader for StreamReader<R> {
         self.reader.read_exact(&mut array)?;
         Ok(size)
     }
+
+    fn offset(&mut self) -> std::result::Result<u64,Error> {
+        self.reader.seek(SeekFrom::Current(0))
+    }
 }
+
+#[cfg(feature="stream")]
+impl<R: Seek + BufRead> Seek for StreamReader<R> {
+        fn seek(&mut self, seek: std::io::SeekFrom) -> std::result::Result<u64, Error> {
+        self.reader.seek(seek)
+    }
+}
\ No newline at end of file
